# This file was generated by liblab | https://liblab.com/

from typing import List, Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.environment.environment import Environment
from ..models.utils.sentinel import SENTINEL
from ..models.utils.cast_models import cast_models
from ..models import HttpValidationError, MenuItem


class MenuitemsService(BaseService):

    @cast_models
    def new(
        self, init: Union[str, None] = SENTINEL, dataset: Union[str, None] = SENTINEL
    ) -> str:
        """Create new dataset (default or specified settings)

        :param init: init, defaults to None
        :type init: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).is_optional().is_nullable().validate(init)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/revision/new",
                [self.get_api_key()],
            )
            .add_query("init", init, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def get(self, name: str, dataset: Union[str, None] = SENTINEL) -> str:
        """Obtain the lastest value for menuitems with specified 'name'

        :param name: name
        :type name: str
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/actual/get",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def set(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        type_: Union[str, None] = SENTINEL,
        target: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all previous values for specified 'name' and add a new value

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        :param target: target, defaults to None
        :type target: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(type_)
        Validator(str).is_optional().is_nullable().validate(target)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/actual/set",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("type", type_, nullable=True)
            .add_query("target", target, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def add(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        type_: Union[str, None] = SENTINEL,
        target: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Add a new value for specified 'name'

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        :param target: target, defaults to None
        :type target: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(type_)
        Validator(str).is_optional().is_nullable().validate(target)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/actual/add",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("type", type_, nullable=True)
            .add_query("target", target, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def all(
        self, name: Union[str, None] = SENTINEL, dataset: Union[str, None] = SENTINEL
    ) -> List[MenuItem]:
        """Obtain a list of all menuitems with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: List[MenuItem]
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/all/get",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return [MenuItem._unmap(item) for item in response]

    @cast_models
    def update(
        self, request_body: List[MenuItem], dataset: Union[str, None] = SENTINEL
    ) -> any:
        """Remove previously set and add new menuitems with specified 'name' fileds with values from 'values' fileds of provided list

        :param request_body: The request body.
        :type request_body: List[MenuItem]
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(MenuItem).is_array().validate(request_body)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/all/update",
                [self.get_api_key()],
            )
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def remove(
        self,
        name: Union[str, None] = SENTINEL,
        value: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all values for specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/all/remove",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("value", value, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("DELETE")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def display(
        self,
        name: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
        format: Union[str, None] = SENTINEL,
    ) -> any:
        """Display a list of all menuitems with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        :param format: format, defaults to None
        :type format: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)
        Validator(str).is_optional().is_nullable().validate(format)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/all/display",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_query("format", format, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def revisions(self) -> any:
        """List of all avaliable revisions (aka 'dataset')

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/menuitems/rev/list",
                [self.get_api_key()],
            )
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def new_1(
        self, init: Union[str, None] = SENTINEL, dataset: Union[str, None] = SENTINEL
    ) -> str:
        """Create new dataset (default or specified settings)

        :param init: init, defaults to None
        :type init: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).is_optional().is_nullable().validate(init)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/revision/new",
                [self.get_api_key()],
            )
            .add_query("init", init, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def get_1(self, name: str, dataset: Union[str, None] = SENTINEL) -> str:
        """Obtain the lastest value for menuitems with specified 'name'

        :param name: name
        :type name: str
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: str
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/actual/get",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def set_1(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        type_: Union[str, None] = SENTINEL,
        target: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all previous values for specified 'name' and add a new value

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        :param target: target, defaults to None
        :type target: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(type_)
        Validator(str).is_optional().is_nullable().validate(target)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/actual/set",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("type", type_, nullable=True)
            .add_query("target", target, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def add_1(
        self,
        name: str,
        value: Union[str, None] = SENTINEL,
        type_: Union[str, None] = SENTINEL,
        target: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Add a new value for specified 'name'

        :param name: name
        :type name: str
        :param value: value, defaults to None
        :type value: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        :param target: target, defaults to None
        :type target: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(type_)
        Validator(str).is_optional().is_nullable().validate(target)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/actual/add",
                [self.get_api_key()],
            )
            .add_query("name", name)
            .add_query("value", value, nullable=True)
            .add_query("type", type_, nullable=True)
            .add_query("target", target, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def all_1(
        self, name: Union[str, None] = SENTINEL, dataset: Union[str, None] = SENTINEL
    ) -> List[MenuItem]:
        """Obtain a list of all menuitems with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: List[MenuItem]
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/all/get",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return [MenuItem._unmap(item) for item in response]

    @cast_models
    def update_1(
        self, request_body: List[MenuItem], dataset: Union[str, None] = SENTINEL
    ) -> any:
        """Remove previously set and add new menuitems with specified 'name' fileds with values from 'values' fileds of provided list

        :param request_body: The request body.
        :type request_body: List[MenuItem]
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(MenuItem).is_array().validate(request_body)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/all/update",
                [self.get_api_key()],
            )
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def remove_1(
        self,
        name: Union[str, None] = SENTINEL,
        value: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
    ) -> any:
        """Remove all values for specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(value)
        Validator(str).is_optional().is_nullable().validate(dataset)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/all/remove",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("value", value, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("DELETE")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def display_1(
        self,
        name: Union[str, None] = SENTINEL,
        dataset: Union[str, None] = SENTINEL,
        format: Union[str, None] = SENTINEL,
    ) -> any:
        """Display a list of all menuitems with specified 'name'

        :param name: name, defaults to None
        :type name: str, optional
        :param dataset: dataset, defaults to None
        :type dataset: str, optional
        :param format: format, defaults to None
        :type format: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        Validator(str).is_optional().is_nullable().validate(name)
        Validator(str).is_optional().is_nullable().validate(dataset)
        Validator(str).is_optional().is_nullable().validate(format)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/all/display",
                [self.get_api_key()],
            )
            .add_query("name", name, nullable=True)
            .add_query("dataset", dataset, nullable=True)
            .add_query("format", format, nullable=True)
            .add_error(422, HttpValidationError)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response

    @cast_models
    def revisions_1(self) -> any:
        """List of all avaliable revisions (aka 'dataset')

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: any
        """

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/api/v1/api/v1/menuitems/rev/list",
                [self.get_api_key()],
            )
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        return response
